import BbPromise from 'bluebird';
import AWS from 'aws-sdk';
import Serverless from 'serverless';
import Plugin, { Logging } from 'serverless/classes/Plugin';
import { DynamoDBTableProps } from '@squareball/cfntypes';
import Service from 'serverless/classes/Service';

type Resources = Service["resources"];
type Command = Plugin.Commands[keyof Plugin.Commands];
type Options = NonNullable<Command["options"]>;
type ConvertType<Type extends string> = Type extends "string" ? string : Type extends "boolean" ? boolean : Type extends "number" ? number : never;
type ExtractOptionType<Option extends Options[keyof Options]> = Option extends {
    type: string;
    required?: boolean;
} ? ConvertType<Option["type"]> | (Option["required"] extends true ? unknown : undefined) : never;
type PluginCommands<P extends Plugin> = P["commands"] extends NonNullable<Plugin.Commands> ? P["commands"] : never;
type PluginSubcommands<P extends Plugin, C extends keyof PluginCommands<P> = keyof PluginCommands<P>> = PluginCommands<P>[C]["commands"];
type PluginAllCommandOptions<P extends Plugin> = {
    [C in keyof PluginCommands<P>]: {
        [SC in keyof PluginSubcommands<P>]: PluginSubcommands<P>[SC] extends {
            options: Options;
        } ? {
            [SCO in keyof PluginSubcommands<P>[SC]["options"]]: ExtractOptionType<PluginSubcommands<P>[SC]["options"][SCO]>;
        } : never;
    };
};
type PluginCommandOptions<P extends Plugin, C extends keyof PluginCommands<P> = keyof PluginCommands<P>> = PluginAllCommandOptions<P>[C];
type PluginSubcommandOptions<P extends Plugin, C extends keyof PluginCommands<P> = keyof PluginCommands<P>, SC extends keyof PluginSubcommands<P, C> = keyof PluginSubcommands<P, C>> = PluginCommandOptions<P, C>[SC];
type ServerlessDynamodbLocalOptions = PluginSubcommandOptions<ServerlessDynamodbLocal, "dynamodb", "start"> & PluginSubcommandOptions<ServerlessDynamodbLocal, "dynamodb", "install">;
type PluginConfig = {
    stages?: string[];
    region?: string;
    seed?: {
        [category: string]: {
            sources: {
                table: string;
                sources: string[];
                rawsources?: string[];
            }[];
        };
    };
    start: ServerlessDynamodbLocalOptions;
};
declare class ServerlessDynamodbLocal implements Plugin {
    protected serverless: Serverless;
    protected logging: Logging;
    config: PluginConfig;
    service: Service;
    options: Serverless.Options & ServerlessDynamodbLocalOptions;
    provider: string;
    hooks: {
        "dynamodb:migrate:migrateHandler": () => Promise<(AWS.DynamoDB.CreateTableInput | undefined)[]> | Promise<void>;
        "dynamodb:seed:seedHandler": () => Promise<void> | BbPromise<({
            table: string;
            sources: string[];
            rawsources?: string[] | undefined;
        } | undefined)[]>;
        "dynamodb:remove:removeHandler": () => BbPromise<unknown>;
        "dynamodb:install:installHandler": () => BbPromise<unknown>;
        "dynamodb:start:startHandler": () => Promise<void>;
        "before:offline:start:init": () => Promise<void>;
        "before:offline:start:end": () => void;
    };
    commands: {
        readonly dynamodb: {
            readonly commands: {
                readonly migrate: {
                    readonly lifecycleEvents: readonly ["migrateHandler"];
                    readonly usage: "Creates local DynamoDB tables from the current Serverless configuration";
                    readonly options: {
                        readonly online: {
                            readonly shortcut: "o";
                            readonly usage: "Uses online tables (default is offline)";
                            readonly type: "boolean";
                        };
                        readonly convertEmptyValues: {
                            readonly shortcut: "e";
                            readonly usage: "Set to true if you would like the document client to convert empty values (0-length strings, binary buffers, and sets) to be converted to NULL types when persisting to DynamoDB.";
                            readonly type: "boolean";
                        };
                    };
                };
                readonly start: {
                    readonly lifecycleEvents: readonly ["startHandler"];
                    readonly usage: "Starts local DynamoDB";
                    readonly options: {
                        readonly port: {
                            readonly shortcut: "p";
                            readonly usage: "The port number that DynamoDB will use to communicate with your application. If you do not specify this option, the default port is 8000";
                            readonly type: "string";
                        };
                        readonly cors: {
                            readonly shortcut: "c";
                            readonly usage: "Enable CORS support (cross-origin resource sharing) for JavaScript. You must provide a comma-separated \"allow\" list of specific domains. The default setting for -cors is an asterisk (*), which allows public access.";
                            readonly type: "string";
                        };
                        readonly inMemory: {
                            readonly shortcut: "i";
                            readonly usage: "DynamoDB; will run in memory, instead of using a database file. When you stop DynamoDB;, none of the data will be saved. Note that you cannot specify both -dbPath and -inMemory at once.";
                            readonly type: "boolean";
                        };
                        readonly dbPath: {
                            readonly shortcut: "d";
                            readonly usage: "The directory where DynamoDB will write its database file. If you do not specify this option, the file will be written to the current directory. Note that you cannot specify both -dbPath and -inMemory at once. For the path, current working directory is <projectroot>/node_modules/serverless-dynamodb-local/dynamob. For example to create <projectroot>/node_modules/serverless-dynamodb-local/dynamob/<mypath> you should specify -d <mypath>/ or --dbPath <mypath>/ with a forwardslash at the end.";
                            readonly type: "string";
                        };
                        readonly sharedDb: {
                            readonly shortcut: "h";
                            readonly usage: "DynamoDB will use a single database file, instead of using separate files for each credential and region. If you specify -sharedDb, all DynamoDB clients will interact with the same set of tables regardless of their region and credential configuration.";
                            readonly type: "boolean";
                        };
                        readonly delayTransientStatuses: {
                            readonly shortcut: "t";
                            readonly usage: "Causes DynamoDB to introduce delays for certain operations. DynamoDB can perform some tasks almost instantaneously, such as create/update/delete operations on tables and indexes; however, the actual DynamoDB service requires more time for these tasks. Setting this parameter helps DynamoDB simulate the behavior of the Amazon DynamoDB web service more closely. (Currently, this parameter introduces delays only for global secondary indexes that are in either CREATING or DELETING status.";
                            readonly type: "boolean";
                        };
                        readonly optimizeDbBeforeStartup: {
                            readonly shortcut: "o";
                            readonly usage: "Optimizes the underlying database tables before starting up DynamoDB on your computer. You must also specify -dbPath when you use this parameter.";
                            readonly type: "boolean";
                        };
                        readonly migrate: {
                            readonly shortcut: "m";
                            readonly usage: "After starting dynamodb local, create DynamoDB tables from the current serverless configuration.";
                            readonly type: "boolean";
                        };
                        readonly seed: {
                            readonly shortcut: "s";
                            readonly usage: "After starting and migrating dynamodb local, injects seed data into your tables. The --seed option determines which data categories to onload.";
                            readonly type: "string";
                        };
                        readonly heapInitial: {
                            readonly usage: "The initial heap size. Specify megabytes, gigabytes or terabytes using m, b, t. E.g., \"2m\"";
                            readonly type: "string";
                        };
                        readonly heapMax: {
                            readonly usage: "The maximum heap size. Specify megabytes, gigabytes or terabytes using m, b, t. E.g., \"2m\"";
                            readonly type: "string";
                        };
                        readonly convertEmptyValues: {
                            readonly shortcut: "e";
                            readonly usage: "Set to true if you would like the document client to convert empty values (0-length strings, binary buffers, and sets) to be converted to NULL types when persisting to DynamoDB.";
                            readonly type: "boolean";
                        };
                        readonly docker: {
                            readonly usage: "Run DynamoDB inside docker container instead of as a local Java program";
                            readonly type: "boolean";
                        };
                        readonly host: {
                            readonly usage: "The host name to use when connecting to DynamoDB. Defaults to localhost.";
                            readonly type: "string";
                        };
                        readonly online: {
                            readonly shortcut: "o";
                            readonly usage: "Uses online tables for migration and seed (default is offline)";
                            readonly type: "boolean";
                        };
                        readonly noStart: {
                            readonly shortcut: "n";
                            readonly usage: "Do not start DynamoDB local (in case it is already running)";
                            readonly type: "boolean";
                        };
                    };
                };
                readonly remove: {
                    readonly lifecycleEvents: readonly ["removeHandler"];
                    readonly usage: "Removes local DynamoDB";
                };
                readonly install: {
                    readonly usage: "Installs local DynamoDB";
                    readonly lifecycleEvents: readonly ["installHandler"];
                    readonly options: {
                        readonly localPath: {
                            readonly shortcut: "x";
                            readonly usage: "Local dynamodb install path";
                            readonly type: "string";
                        };
                    };
                };
            };
        };
    };
    constructor(serverless: Serverless, options: Serverless.Options & ServerlessDynamodbLocalOptions, logging: Logging);
    get port(): number;
    get host(): string;
    get stage(): string;
    shouldExecute(): boolean;
    dynamodbOptions(options?: {
        online?: boolean;
        convertEmptyValues?: boolean;
        region: string | null;
    }): {
        raw: AWS.DynamoDB;
        doc: AWS.DynamoDB.DocumentClient;
    };
    migrateHandler(): Promise<(AWS.DynamoDB.CreateTableInput | undefined)[]> | Promise<void>;
    seedHandler(): Promise<void> | BbPromise<({
        table: string;
        sources: string[];
        rawsources?: string[] | undefined;
    } | undefined)[]>;
    removeHandler(): BbPromise<unknown>;
    installHandler(): BbPromise<unknown>;
    startHandler(): Promise<void>;
    endHandler(): void;
    getDefaultStack(): {
        Resources: {
            [key: string]: any;
        };
    } | {
        [key: string]: any;
    };
    getAdditionalStacks(): any[];
    hasAdditionalStacksPlugin(): boolean;
    getTableDefinitionsFromStack(stack?: Resources): any[];
    get tables(): any[];
    get seedSources(): ({
        table: string;
        sources: string[];
        rawsources?: string[] | undefined;
    } | undefined)[];
    createTable(dynamodb: {
        raw: AWS.DynamoDB;
        doc: AWS.DynamoDB.DocumentClient;
    }, cfnTable: DynamoDBTableProps): Promise<AWS.DynamoDB.CreateTableInput | undefined>;
}

export { ServerlessDynamodbLocal as default };
